(define start-swl-server
  (lambda (args)
    
    (import swl:foreign)
    (import swl:macros)
    (import swl:module-setup)
    
   ; It appears that clients may only connect via the specified address,
   ; so this prevents remote access to our SWL server.  (An attacker could
   ; set up a proxy to forward connections to the local machine.)
    (define hostip "127.0.0.1")
    
    (define open-swl-server
      (lambda (p)
        (on-error #f
          (let ([cookie (gensym)])
            (define server-port
              (swl:open-tcp-server hostip 0
                (lambda (cp hostname clientport)
                  (when (eq? (read cp) cookie)
                    (let* ([dir (read cp)] [args (read cp)])
                      (swl:begin-application
                        (lambda (token)
                          (define thread
                            (thread-fork
                              (lambda ()
                                (call/cc
                                  (lambda (k)
                                    (parameterize ([current-directory dir]
                                                   [abort-handler k]
                                                   [exit-handler k]
                                                   [interrupt-handler k]
                                                   [reset-handler k]
                                                   [error-handler-continuation k])
                                      "; defined in init.ss"
                                      (do-command-line args))))
                                (set! thread #f)
                                (swl:end-application token))))
                          (lambda ()
                            (when thread
                              (case (warning-dialog #f ; want this dialog to block
                                      (format "Still processing command-line arguments ~s" args)
                                      '(|keep processing| |stop processing|))
                                [(|stop processing|)
                                 (on-error 'ignore
                                   (thread-kill thread)
                                   (set! thread #f))
                                 (swl:end-application token)])))
                          ))))
                  (close-port cp))))
           ; protect from gc
            (#%\#sputprop 'gc-protect '*server-port* server-port)
            (truncate-file p 0)
            (write cookie p)
            (newline p)
            (write ((port-handler server-port) 'port-number p) p)
            (flush-output-port p)))))
    
    (on-error (with-message msg (warning-dialog 'noblock (format "Unable to contact or initialize SWL server.\n\n~a" msg)))
      (let ([ip (case (machine-type)
                  [(i3nt ppcnt)
                   (let ([key (format "HKEY_LOCAL_MACHINE\\Software\\Chez Scheme\\swl~a\\ServerFile" swl:version)])
                     (on-error (error #f "Unable to determine server port.\nCheck registry setting for ~a" key)
                       (open-input-output-file
                         (or (swl:get-registry key #t)
                             (error #f "missing registry entry for ~a" key))
                         '(exclusive mode #o600))))]
                  [else (open-input-output-file
                          (format "/tmp/.swl_~a" (getenv "USER"))
                         '(exclusive mode #o600))])])
        (on-error (open-swl-server ip)
          (let* ([cookie (read ip)] [port-number (read ip)])
            (unless (and (gensym? cookie) (fixnum? port-number) (fx<= 0 port-number #xFFFF))
              (error #f "malformed server info file ~s" ip))
            (let ([p (swl:open-tcp-client hostip port-number #f)])
              (fprintf p "~s ~s ~s" cookie (current-directory) args)
              (close-port p)
              (close-port ip)
              (exit 0))))
        (close-port ip)))
))

#!eof

(import swl:foreign)
(import swl:macros)
(import swl:threads)

(import swl:module-setup)

; what if we have a separate app startup queue...
; isn't this what begin application is?

(define theq
  (let ([q (thread-make-msg-queue 'startup)])
(define n 0)
    (thread-fork
      (rec loop
        (lambda ()
          (let ([thunk (thread-receive-msg q)])
(set! n (+ n 1))
(printf "Got a new thunk to evaluate n=~s\n" n)
            (thread-fork thunk)
            (loop)))))
    q))

(define n 0)
(define server
  (swl:open-tcp-server "localhost" 3030
    (lambda (cp hostname num)
      (printf "connection from ~s on ~s\n" hostname num)
      (set! n (+ n 1))
      (case (read cp)
        [(new-repl)
         (set! e1 (error-handler))
         (swl:begin-application
           (lambda (token)
             (set! e2 (error-handler))
             (new-repl)
             (swl:end-application token)
             void))])
      (write n cp)
      (close-port cp))))

(define connect
  (lambda ()
    (let ([p (swl:open-tcp-client "localhost" 3030 #f)])
      (write 'new-repl p)
      (newline p)
      (flush-output-port p)
      (printf "client connected with n=~s at ~s\n"
        (read p) ((port-handler p) 'port-number p)))))

#!eof ----------------------------------------------------------------------------

(define server
  (swl:open-tcp-server "localhost" 3030
    (lambda (cp hostname num)
      (printf "connection from ~s on ~s\n" hostname num)
      (set! n (+ n 1))
      (case (read cp)
        [(new-repl)
(set! e1 (error-handler))
#;
; just running the code here breaks
(new-repl)
#;
; sending it off to another thread seems to work
(thread-send-msg theq (lambda () (set! e2 (error-handler)) (new-repl)))
;#;
; so we'd expect swl:begin-application to work as well
;  but this fails apparently because server's reset handler is unassigned

; Aha! the problem is that begin-application is grabbing a mutex and this parameterizes
; the error handler...

(swl:begin-application
  (lambda (token)
(set! e2 (error-handler))
        (new-repl)
    (swl:end-application token)
    void))

])
      (write n cp)
      (close-port cp))))

#!eof ----------------------------------------------------------------------------

(define n 0)
(define server
  (swl:open-tcp-server "localhost" 3030
    (lambda (cp hostname num)
      (printf "connection from ~s on ~s\n" hostname num)
      (set! n (+ n 1))
(swl:begin-application
(lambda (token)
(case (read cp) [(new-repl) (new-repl (lambda (r) (swl:end-application token)) void)])
void
))
      (write n cp)
      (close-port cp))))

(define connect
  (lambda ()
    (let ([p (swl:open-tcp-client "localhost" 3030 #f)])
      (write 'new-repl p)
      (newline p)
      (flush-output-port p)
      (printf "client connected with n=~s at ~s\n"
        (read p) ((port-handler p) 'port-number p)))))

#|

; in the newly forked repl, try doing:

  (display "hi\n")

and then hit up arrow or Control-p to go into the history...

somehow the begin-exp mark hasn't been set yet by the time we get there.

or try causing error, e.g. unbound variable error and we get
Warning: server's reset handler unassigned:  thread-num=24 thread-name="Repl"

 * also looks like the forked repl has priority of -1 where others have normal priority
   --> maybe the system thread has whacked the reset handler and done the thread-become server stuff...

|#
